from prompt_toolkit.filters import Condition


class KeyBindings:

    edit_mode: bool
    help_mode: bool

    def bind_keys(self):
        @Condition
        def edit_mode() -> bool:
            return self.edit_mode and not self.help_mode

        @Condition
        def command_mode() -> bool:
            return not self.edit_mode and not self.help_mode

        @Condition
        def help_mode() -> bool:
            return self.help_mode

        @Condition
        def not_help_mode() -> bool:
            return not self.help_mode

        @self.key_bindings.add("enter", filter=help_mode)
        @self.key_bindings.add("c-q", filter=help_mode)
        @self.key_bindings.add("escape", filter=help_mode)
        def quit_help(event):
            self.quitting = False
            self.quit_help()

        @self.key_bindings.add("up", filter=help_mode)
        def scroll_help_up(event):
            self.scroll_help_up()

        @self.key_bindings.add("down", filter=help_mode)
        def scroll_help_down(event):
            self.scroll_help_down()

        @self.key_bindings.add("c-h", filter=command_mode)
        def c_h(event):
            self.quitting = False
            self.show_help()

        @self.key_bindings.add("c-q", filter=not_help_mode)
        async def c_q(event):
            await self.exit()

        @self.key_bindings.add("c-s", filter=command_mode)
        def c_s(event):
            self.quitting = False
            self.save()

        @self.key_bindings.add("enter", filter=command_mode)
        def enter_cell(event):
            self.quitting = False
            self.enter_cell()

        @self.key_bindings.add("escape", filter=edit_mode, eager=True)
        def escape(event):
            self.quitting = False
            self.exit_cell()

        @self.key_bindings.add("up", filter=command_mode)
        def up(event):
            self.quitting = False
            self.go_up()

        @self.key_bindings.add("down", filter=command_mode)
        def down(event):
            self.quitting = False
            self.go_down()

        @self.key_bindings.add("c-up", filter=command_mode)
        def c_up(event):
            self.quitting = False
            self.move_up()

        @self.key_bindings.add("c-down", filter=command_mode)
        def c_down(event):
            self.quitting = False
            self.move_down()

        @self.key_bindings.add("l", filter=command_mode)
        def l(event):  # noqa
            self.quitting = False
            self.clear_output()

        @self.key_bindings.add("m", filter=command_mode)
        def m(event):
            self.quitting = False
            self.markdown_cell()

        @self.key_bindings.add("o", filter=command_mode)
        def o(event):
            self.quitting = False
            self.code_cell()

        @self.key_bindings.add("c-e", filter=command_mode)
        async def c_e(event):
            self.quitting = False
            await self.queue_run_cell()

        @self.key_bindings.add("c-r", filter=command_mode)
        async def c_r(event):
            self.quitting = False
            await self.queue_run_cell(and_select_below=True)

        @self.key_bindings.add("x", filter=command_mode)
        def x(event):
            self.quitting = False
            self.cut_cell()

        @self.key_bindings.add("c", filter=command_mode)
        def c(event):
            self.quitting = False
            self.copy_cell()

        @self.key_bindings.add("c-v", filter=command_mode)
        def c_v(event):
            self.quitting = False
            self.paste_cell()

        @self.key_bindings.add("v", filter=command_mode)
        def v(event):
            self.quitting = False
            self.paste_cell(below=True)

        @self.key_bindings.add("a", filter=command_mode)
        def a(event):
            self.quitting = False
            self.insert_cell()

        @self.key_bindings.add("b", filter=command_mode)
        def b(event):
            self.quitting = False
            self.insert_cell(below=True)



import asyncio
from typing import Optional

import typer

from nbterm import __version__
from .notebook import Notebook


def version_callback(value: bool):
    if value:
        typer.echo(f"nbterm {__version__}")
        raise typer.Exit()


def main(
    notebook_path: str = typer.Argument("", help="Path to the notebook."),
    no_kernel: Optional[bool] = typer.Option(
        None, "--no-kernel", help="Don't launch a kernel."
    ),
    run: Optional[bool] = typer.Option(None, "--run", help="Run the notebook."),
    save_path: Optional[str] = typer.Option(
        None, "--save-path", help="Path to save the notebook."
    ),
    version: Optional[bool] = typer.Option(
        None, "--version", callback=version_callback, help="Show the version and exit."
    ),
):
    nb = Notebook(notebook_path, no_kernel=no_kernel or False, save_path=save_path)
    if run:
        assert no_kernel is not True
        asyncio.run(nb.run_all())
        if save_path is None:
            i = nb.nb_path.rfind(".")
            save_path = nb.nb_path[:i] + "_run" + nb.nb_path[i:]
        nb.save(save_path)
        typer.echo(f"Executed notebook has been saved to: {save_path}")
    else:
        nb.show()


def cli():
    typer.run(main)


if __name__ == "__main__":
    cli()



import os
import itertools
import asyncio
from typing import List, Dict, Any, Optional

from prompt_toolkit import ANSI
from prompt_toolkit.key_binding import KeyBindings as PtKeyBindings
from prompt_toolkit.layout import ScrollablePane
from prompt_toolkit.layout.containers import HSplit, VSplit
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.layout.controls import FormattedTextControl
from prompt_toolkit.lexers import PygmentsLexer
from prompt_toolkit.widgets.toolbars import FormattedTextToolbar
from pygments.lexers.python import PythonLexer  # type: ignore
from pygments.lexers.c_cpp import CppLexer  # type: ignore
from prompt_toolkit import Application
from rich.console import Console
import kernel_driver  # type: ignore
from kernel_driver import KernelDriver

from .cell import (
    Cell,
    ONE_COL,
    set_console,
    rich_print,
    get_output_text_and_height,
)
from .help import Help
from .format import Format
from .key_bindings import KeyBindings


class Notebook(Help, Format, KeyBindings):

    app: Optional[Application]
    layout: Layout
    copied_cell: Optional[Cell]
    console: Console
    _run_notebook_nb_path: str
    cells: List[Cell]
    executing_cells: List[Cell]
    json: Dict[str, Any]
    kd: Optional[KernelDriver]
    execution_count: int
    current_cell_idx: int
    idle: Optional[asyncio.Event]
    lexer: Optional[PygmentsLexer] = PygmentsLexer(PythonLexer)
    language: str
    kernel_name: str
    no_kernel: bool
    save_path: Optional[str]
    dirty: bool
    quitting: bool
    kernel_busy: bool
    kernel_cwd: str

    def __init__(
        self, nb_path: str, no_kernel: bool = False, save_path: Optional[str] = None
    ):
        if nb_path:
            if os.path.isdir(nb_path):
                self.kernel_cwd = os.path.abspath(nb_path)
                self.nb_path = ""
            else:
                self.kernel_cwd = os.path.abspath(os.path.dirname(nb_path))
                assert os.path.isdir(self.kernel_cwd)
                self.nb_path = os.path.relpath(
                    os.path.abspath(nb_path), self.kernel_cwd
                )
            os.chdir(self.kernel_cwd)
        else:
            self.kernel_cwd = os.getcwd()
            self.nb_path = ""
        self.app = None
        self.copied_cell = None
        self.console = Console()
        set_console(self.console)
        self.save_path = save_path
        self.no_kernel = no_kernel
        self.executing_cells = []
        if os.path.isfile(self.nb_path):
            self.read_nb()
        else:
            self.create_nb()
        self.dirty = False
        self.quitting = False
        self.kernel_busy = False
        self.execution_count = 1
        self.current_cell_idx = 0
        self.idle = None
        self.edit_mode = False
        self.help_mode = False

    def set_language(self):
        self.kernel_name = self.json["metadata"]["kernelspec"]["name"]
        if self.kernel_name.startswith("python"):
            self.lexer = PygmentsLexer(PythonLexer)
            self.language = "python"
        elif self.kernel_name.startswith("xcpp"):
            self.lexer = PygmentsLexer(CppLexer)
            self.language = "cpp"
        else:
            self.lexer = None
            self.language = ""
        if self.no_kernel:
            self.kd = None
        else:
            try:
                self.kd = KernelDriver(kernel_name=self.kernel_name, log=False)
                kernel_driver.driver._output_hook_default = self.output_hook
            except RuntimeError:
                self.kd = None

    @property
    def current_cell(self):
        return self.cells[self.current_cell_idx]

    async def run_cell(self, idx: Optional[int] = None):
        if idx is None:
            idx = self.current_cell_idx
        self.focus(idx)
        self.executing_cells = [self.current_cell]
        await self.current_cell.run()

    async def run_all(self):
        await self.kd.start()
        for i in range(len(self.cells)):
            await self.run_cell(i)

    def show(self):
        self.key_bindings = PtKeyBindings()
        self.bind_keys()
        self.create_layout()
        self.app = Application(
            layout=self.layout, key_bindings=self.key_bindings, full_screen=True
        )
        self.focus(0)
        asyncio.run(self._show())

    def update_layout(self, idx: Optional[int] = None):
        if self.app:
            self.create_layout()
            self.app.layout = self.layout
        if idx is None:
            idx = self.current_cell_idx
        self.focus(idx)

    def create_layout(self):
        inout_cells = list(
            itertools.chain.from_iterable(
                [
                    (
                        VSplit([cell.input_prefix, cell.input]),
                        VSplit([cell.output_prefix, ONE_COL, cell.output, ONE_COL]),
                    )
                    for cell in self.cells
                ]
            )
        )
        nb_window = ScrollablePane(HSplit(inout_cells))

        def get_top_bar_text():
            text = ""
            if self.dirty:
                text += "+ "
            text += os.path.basename(self.nb_path)
            if self.dirty and self.quitting:
                text += (
                    " (no write since last change, please exit again to confirm, "
                    "or save your changes)"
                )
            return text

        def get_bottom_bar_text():
            text = ""
            if not self.no_kernel and self.kernel_name:
                kernel_status = ["idle", "busy"][self.kernel_busy]
                text += f"{self.kernel_name} ({kernel_status}) at "
            text += self.kernel_cwd
            return text

        self.top_bar = FormattedTextToolbar(
            get_top_bar_text, style="#ffffff bg:#444444"
        )
        self.bottom_bar = FormattedTextToolbar(
            get_bottom_bar_text, style="#ffffff bg:#444444"
        )
        root_container = HSplit([self.top_bar, nb_window, self.bottom_bar])
        self.layout = Layout(root_container)

    def focus(self, idx: int):
        if 0 <= idx < len(self.cells):
            if self.app:
                self.app.layout.focus(self.cells[idx].input_window)
            self.current_cell_idx = idx

    def exit_cell(self):
        self.edit_mode = False
        self.current_cell.update_json()
        self.current_cell.set_input_readonly()

    def enter_cell(self):
        self.edit_mode = True
        self.current_cell.set_input_editable()

    def move_up(self):
        idx = self.current_cell_idx
        if idx > 0:
            self.dirty = True
            self.cells[idx - 1], self.cells[idx] = self.cells[idx], self.cells[idx - 1]
            self.update_layout(idx - 1)

    def move_down(self):
        idx = self.current_cell_idx
        if idx < len(self.cells) - 1:
            self.dirty = True
            self.cells[idx], self.cells[idx + 1] = self.cells[idx + 1], self.cells[idx]
            self.update_layout(idx + 1)

    def clear_output(self):
        self.current_cell.clear_output()

    def markdown_cell(self):
        self.current_cell.set_as_markdown()

    def code_cell(self):
        self.current_cell.set_as_code()

    async def queue_run_cell(self, and_select_below: bool = False):
        if self.kd:
            self.executing_cells.append(self.current_cell)
            if and_select_below:
                if self.current_cell_idx == len(self.cells) - 1:
                    self.insert_cell(self.current_cell_idx + 1)
                self.focus(self.current_cell_idx + 1)
            await self.executing_cells[-1].run()

    def cut_cell(self, idx: Optional[int] = None):
        self.dirty = True
        if idx is None:
            idx = self.current_cell_idx
        self.copied_cell = self.cells.pop(idx)
        if not self.cells:
            self.cells = [Cell(self)]
        elif idx == len(self.cells):
            idx -= 1
        self.update_layout(idx)

    def copy_cell(self, idx: Optional[int] = None):
        if idx is None:
            idx = self.current_cell_idx
        idx = self.current_cell_idx
        self.copied_cell = self.cells[idx]

    def paste_cell(self, idx: Optional[int] = None, below=False):
        self.dirty = True
        if idx is None:
            idx = self.current_cell_idx + below
        if self.copied_cell is not None:
            pasted_cell = self.copied_cell.copy()
            self.cells.insert(idx, pasted_cell)
            self.update_layout(idx)

    def insert_cell(self, idx: Optional[int] = None, below=False):
        self.dirty = True
        if idx is None:
            idx = self.current_cell_idx + below
        self.cells.insert(idx, Cell(self))
        self.update_layout(idx)

    def output_hook(self, msg: Dict[str, Any]):
        msg_type = msg["header"]["msg_type"]
        content = msg["content"]
        outputs = self.executing_cells[0].json["outputs"]
        if msg_type == "stream":
            if (not outputs) or (outputs[-1]["name"] != content["name"]):
                outputs.append(
                    {"name": content["name"], "output_type": msg_type, "text": []}
                )
            outputs[-1]["text"].append(content["text"])
        elif msg_type in ("display_data", "execute_result"):
            outputs.append(
                {
                    "data": {"text/plain": [content["data"].get("text/plain", "")]},
                    "execution_count": self.execution_count,
                    "metadata": {},
                    "output_type": msg_type,
                }
            )
            text = rich_print(f"Out[{self.execution_count}]:", style="red", end="")
            self.executing_cells[0].output_prefix.content = FormattedTextControl(
                text=ANSI(text)
            )
        elif msg_type == "error":
            outputs.append(
                {
                    "ename": content["ename"],
                    "evalue": content["evalue"],
                    "output_type": "error",
                    "traceback": content["traceback"],
                }
            )
        else:
            return
        text, height = get_output_text_and_height(outputs)
        self.executing_cells[0].output.content = FormattedTextControl(text=text)
        self.executing_cells[0].output.height = height
        if self.app:
            self.app.invalidate()

    async def _show(self):
        if self.kd:
            asyncio.create_task(self.kd.start())
        await self.app.run_async()

    async def exit(self):
        if self.dirty and not self.quitting:
            self.quitting = True
            return
        if self.kd:
            await self.kd.stop()
        self.app.exit()

    def go_up(self):
        self.focus(self.current_cell_idx - 1)

    def go_down(self):
        self.focus(self.current_cell_idx + 1)



import json
from typing import Optional

from .cell import Cell


class Format:

    nb_path: str

    def read_nb(self) -> None:
        with open(self.nb_path) as f:
            self.json = json.load(f)
        self.set_language()  # type: ignore
        self.cells = [
            Cell(self, cell_json=cell_json) for cell_json in self.json["cells"]
        ]
        del self.json["cells"]

    def save(self, path: Optional[str] = None) -> None:
        self.dirty = False
        path = path or self.save_path or self.nb_path  # type: ignore
        if path:
            nb_json = {"cells": [cell.json for cell in self.cells]}
            nb_json.update(self.json)
            with open(path, "wt") as f:
                json.dump(nb_json, f, indent=1)
                f.write("\n")

    def create_nb(self) -> None:
        self.json = {
            "metadata": {
                "kernelspec": {
                    "display_name": "Python 3",
                    "language": "python",
                    "name": "python3",
                },
                "language_info": {
                    "file_extension": ".py",
                    "mimetype": "text/x-python",
                    "name": "python",
                    "version": "3.9.2",
                },
            },
            "nbformat": 4,
            "nbformat_minor": 4,
        }
        self.set_language()  # type: ignore
        self.cells = [Cell(self)]



from rich.markdown import Markdown
from prompt_toolkit.layout.containers import Window
from prompt_toolkit.layout.controls import FormattedTextControl
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit import ANSI

from .cell import rich_print

md = Markdown(
    "## nbterm help\n"
    "There are two modes: edit mode, and command mode.\n"
    "\n"
    "- `enter`: enter the edit mode, allowing to type into the cell.\n"
    "- `esc`: exit the edit mode and enter the command mode.\n"
    "\n"
    "In command mode:\n"
    "\n"
    "- `up`: select cell above.\n"
    "- `down`: select cell above.\n"
    "- `ctrl-up`: move cell up.\n"
    "- `ctrl-down`: move cell down.\n"
    "- `a`: insert cell above.\n"
    "- `b`: insert cell below.\n"
    "- `x`: cut the cell.\n"
    "- `c`: copy the cell.\n"
    "- `ctrl-v`: paste cell above.\n"
    "- `v`: paste cell below.\n"
    "- `o`: set as code cell.\n"
    "- `m`: set as Markdown cell.\n"
    "- `l`: clear cell outputs.\n"
    "- `ctrl-e`: run cell.\n"
    "- `ctrl-r`: run cell and select below.\n"
    "- `ctrl-s`: save.\n"
    "- `ctrl-q`: exit.\n"
    "- `ctrl-h`: show help.\n"
)


class Help:

    help_mode: bool
    help_text: str
    help_window: Window
    help_line: int

    def show_help(self):
        self.help_mode = True
        self.help_text = rich_print(md)
        self.help_window = Window(
            content=FormattedTextControl(text=ANSI(self.help_text))
        )
        self.app.layout = Layout(self.help_window)
        self.help_line = 0

    def scroll_help_up(self):
        if self.help_line > 0:
            self.help_line -= 1
            text = "\n".join(self.help_text.split("\n")[self.help_line :])  # noqa
            self.help_window.content = FormattedTextControl(text=ANSI(text))

    def scroll_help_down(self):
        if self.help_line < self.help_text.count("\n"):
            self.help_line += 1
            text = "\n".join(self.help_text.split("\n")[self.help_line :])  # noqa
            self.help_window.content = FormattedTextControl(text=ANSI(text))

    def quit_help(self):
        self.help_mode = False
        self.update_layout()
        self.help_text = rich_print(md, self.console)
        self.help_window = Window(
            content=FormattedTextControl(text=ANSI(self.help_text))
        )



import asyncio
import copy
from typing import Dict, List, Any, Optional, Union

from prompt_toolkit import ANSI
from prompt_toolkit.buffer import Buffer
from prompt_toolkit.widgets import Frame
from prompt_toolkit.layout.containers import Window, HSplit, VSplit
from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
from rich.syntax import Syntax
from rich.markdown import Markdown
from rich.console import Console


ONE_COL: Window = Window(width=1)
ONE_ROW: Window = Window(height=1)
CONSOLE: Optional[Console] = None


def set_console(console: Console):
    global CONSOLE
    CONSOLE = console


def rich_print(
    string: str, console: Optional[Console] = None, style: str = "", end: str = "\n"
):
    console = console or CONSOLE
    assert console is not None
    with console.capture() as capture:
        console.print(string, style=style, end=end)
    return capture.get()


def get_output_text_and_height(outputs: List[Dict[str, Any]]):
    text_list = []
    height = 0
    for output in outputs:
        if output["output_type"] == "stream":
            text = "".join(output["text"])
            height += text.count("\n") or 1
            if output["name"] == "stderr":
                # TODO: take terminal width into account
                lines = [line + " " * (200 - len(line)) for line in text.split()]
                text = "\n".join(lines)
                text = rich_print(text, style="white on red")
        elif output["output_type"] == "error":
            text = "\n".join(output["traceback"])
            height += text.count("\n") + 1
        elif output["output_type"] == "execute_result":
            text = "\n".join(output["data"].get("text/plain", ""))
            height += text.count("\n") or 1
        else:
            continue

        text_list.append(text)
    text_ansi = ANSI("".join(text_list))
    return text_ansi, height


def empty_cell_json():
    return {
        "cell_type": "code",
        "execution_count": None,
        "metadata": {},
        "source": [],
        "outputs": [],
    }


class Cell:

    input: Union[Frame, HSplit]
    output: Window
    json: Dict[str, Any]
    input_prefix: Window
    output_prefix: Window
    input_window: Window
    input_buffer: Buffer

    def __init__(self, notebook, cell_json: Optional[Dict[str, Any]] = None):
        self.notebook = notebook
        self.json = cell_json or empty_cell_json()
        self.input_prefix = Window(width=10)
        self.output_prefix = Window(width=10, height=0)
        input_text = "".join(self.json["source"])
        if self.json["cell_type"] == "code":
            execution_count = self.json["execution_count"] or " "
            text = rich_print(
                f"\nIn [{execution_count}]:",
                style="green",
                end="",
            )
            self.input_prefix.content = FormattedTextControl(text=ANSI(text))
            outputs = self.json["outputs"]
            for output in outputs:
                if "execution_count" in output:
                    text = rich_print(
                        f"Out[{output['execution_count']}]:",
                        style="red",
                        end="",
                    )
                    self.output_prefix.content = FormattedTextControl(text=ANSI(text))
                    break
        else:
            outputs = []
        output_text, output_height = get_output_text_and_height(outputs)
        self.input_window = Window()
        self.input_buffer = Buffer(on_text_changed=self.input_text_changed)
        self.input_buffer.text = input_text
        self.set_input_readonly()
        if self.json["cell_type"] == "markdown":
            self.input = HSplit(
                [ONE_ROW, VSplit([ONE_COL, self.input_window]), ONE_ROW]
            )
        else:
            self.input = Frame(self.input_window)
        self.output = Window(content=FormattedTextControl(text=output_text))
        self.output.height = output_height

    def copy(self):
        cell_json = copy.deepcopy(self.json)
        cell = Cell(self.notebook, cell_json=cell_json)
        return cell

    def input_text_changed(self, _=None):
        self.notebook.dirty = True
        self.notebook.quitting = False
        line_nb = self.input_buffer.text.count("\n")
        self.input_window.height = line_nb + 1

    def set_as_markdown(self):
        prev_cell_type = self.json["cell_type"]
        if prev_cell_type != "markdown":
            self.notebook.dirty = True
            self.json["cell_type"] = "markdown"
            if "outputs" in self.json:
                del self.json["outputs"]
            if "execution_count" in self.json:
                del self.json["execution_count"]
            self.input_prefix.content = FormattedTextControl(text="")
            self.clear_output()
            self.set_input_readonly()
            if prev_cell_type == "code":
                self.input = HSplit(
                    [ONE_ROW, VSplit([ONE_COL, self.input_window]), ONE_ROW]
                )
                self.notebook.update_layout(self.notebook.current_cell_idx)

    def set_as_code(self):
        prev_cell_type = self.json["cell_type"]
        if prev_cell_type != "code":
            self.notebook.dirty = True
            self.json["cell_type"] = "code"
            self.json["outputs"] = []
            self.json["execution_count"] = None
            text = rich_print("\nIn [ ]:", style="green", end="")
            self.input_prefix.content = FormattedTextControl(text=ANSI(text))
            self.set_input_readonly()
            if prev_cell_type == "markdown":
                self.input = Frame(self.input_window)
                self.notebook.update_layout(self.notebook.current_cell_idx)

    def set_input_readonly(self):
        if self.json["cell_type"] == "markdown":
            text = self.input_buffer.text or "Type *Markdown*"
            md = Markdown(text)
            text = rich_print(md)
        elif self.json["cell_type"] == "code":
            code = Syntax(self.input_buffer.text, self.notebook.language)
            text = rich_print(code)
        self.input_window.content = FormattedTextControl(text=ANSI(text))
        self.input_window.height = text.count("\n") or 1

    def set_input_editable(self):
        if self.json["cell_type"] == "code":
            self.input_window.content = BufferControl(
                buffer=self.input_buffer, lexer=self.notebook.lexer
            )
        else:
            self.input_window.content = BufferControl(buffer=self.input_buffer)
        self.input_window.height = self.input_buffer.text.count("\n") + 1

    def clear_output(self):
        if self.output.height > 0:
            self.notebook.dirty = True
            self.output.height = 0
            self.output.content = FormattedTextControl(text="")
            self.output_prefix.content = FormattedTextControl(text="")
            self.output_prefix.height = 0
            if self.json["cell_type"] == "code":
                self.json["outputs"] = []

    def update_json(self):
        src_list = [line + "\n" for line in self.input_buffer.text.split("\n")]
        src_list[-1] = src_list[-1][:-1]
        self.json["source"] = src_list

    async def run(self):
        if self.notebook.executing_cells[0] is not self:
            self.clear_output()
        if self.json["cell_type"] == "code":
            code = self.input_buffer.text.strip()
            if code:
                self.notebook.dirty = True
                self.notebook.kernel_busy = True
                executing_text = rich_print("\nIn [*]:", style="green", end="")
                if self.notebook.executing_cells[0] is self:
                    already_executing = True
                else:
                    already_executing = False
                    self.input_prefix.content = FormattedTextControl(
                        text=ANSI(executing_text)
                    )
                if self.notebook.idle is None:
                    self.notebook.idle = asyncio.Event()
                else:
                    while True:
                        await self.notebook.idle.wait()
                        if self.notebook.executing_cells[0] is self:
                            break
                    self.notebook.idle.clear()
                if already_executing:
                    self.input_prefix.content = FormattedTextControl(
                        text=ANSI(executing_text)
                    )
                    self.clear_output()
                await self.notebook.kd.execute(self.input_buffer.text)
                self.notebook.kernel_busy = False
                text = rich_print(
                    f"\nIn [{self.notebook.execution_count}]:",
                    style="green",
                    end="",
                )
                self.input_prefix.content = FormattedTextControl(text=ANSI(text))
                self.json["execution_count"] = self.notebook.execution_count
                self.notebook.execution_count += 1
                if self.notebook.app:
                    self.notebook.app.invalidate()
                self.notebook.executing_cells.remove(self)
                self.notebook.idle.set()
            else:
                self.clear_output()
        else:
            self.clear_output()
            self.notebook.executing_cells.remove(self)



import os
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))
version_ns = {}
with open(os.path.join(here, "nbterm", "_version.py")) as f:
    exec(f.read(), {}, version_ns)

setup(
    name="nbterm",
    version=version_ns["__version__"],
    url="https://github.com/davidbrochart/nbterm.git",
    author="David Brochart",
    author_email="david.brochart@gmail.com",
    description="A tool for viewing, editing and executing Jupyter Notebooks in the terminal",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=["nbterm"],
    python_requires=">=3.7",
    install_requires=[
        "prompt-toolkit>=3.0.16",
        "typer",
        "pygments",
        "rich",
        "kernel_driver",
    ],
    extras_require={
        "test": [
            "mypy",
            "flake8",
            "black",
            "pytest",
            "ipykernel",
        ],
    },
    entry_points={
        "console_scripts": ["nbterm = nbterm.nbterm:cli"],
    },
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)
